<?php

namespace App\Http\Controllers\Api\Auth;

use App\Models\User;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Repositories\User\Auth\AuthRepositories;
use App\Http\Requests\Api\Register\RegisterRequest;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use App\Http\Requests\Api\UpdateProfile\UpdateProfileRequest;
use App\Http\Requests\Api\GoogleAuth\GoogleAuthRequest;
use App\Http\Requests\Api\FacebookAuth\FacebookAuthRequest;
use App\Http\Requests\Api\AppleAuth\AppleAuthRequest;
use App\Http\Traits\ApiResponse;
use Carbon\Carbon;
use Spatie\Permission\Traits\HasRoles;
use App\Http\Resources\Frontend\User\Auth\GetUser;

class AuthController extends Controller
{
    use ApiResponse, HasRoles;

    private $authRepository;
    public function __construct(AuthRepositories $authRepository)
    {
        $this->authRepository = $authRepository;
    }

    public function register(Request $request)
    {
        $status_code = 404;
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:8',
            'phone' => 'nullable|string|min:8',
            'gender' => 'required',
            // 'type' => 'required',
            'experience' => 'required_if:type,chef',
            'speciality' => 'required_if:type,chef',
        ], [
            'experience.required_if' => 'The Experience field is required',
            'speciality.required_if' => 'The Speciality field is required',
        ]);
        
        $data['name'] = "";
        $data['email'] = "";
        $data['phone'] = "";
        $data['status'] = 'error';
        $data['message'] = 'Validation error found';
        $data['password'] = "";
        $data['gender'] = "";
        $data['image'] = "";
        $data['type'] = "";
        $data['experience'] = "";
        $data['speciality'] = "";

        if ($validator->fails()) {
            $status_code = 400;
            $error_message = $validator->messages();
            if ($error_message->first('name')) 
            {
                $data['name'] = $error_message->first('name');
            }
            if ($error_message->first('email')) 
            {
                $data['email'] = $error_message->first('email');
            }
            if ($error_message->first('password')) 
            {
                $data['password'] = $error_message->first('password');
            }
            
            if ($error_message->first('phone')) 
            {
                $data['phone'] = $error_message->first('phone');
            }
            if ($error_message->first('gender')) 
            {
                $data['gender'] = $error_message->first('gender');
            }
            // if ($error_message->first('type')) 
            // {
            //     $data['type'] = $error_message->first('type');
            // }
            if ($error_message->first('experience')) 
            {
                $data['experience'] = $error_message->first('experience');
            }
            if ($error_message->first('speciality')) 
            {
                $data['speciality'] = $error_message->first('speciality');
            }
            return response()->json($data, $status_code);
        }

        // Rest of your function remains the same...
        $data['name'] = $request->name ?? "";
        $data['email'] = $request->email;
        $data['phone'] = $request->phone;
        $data['gender'] = $request->gender;
        $data['type'] = $request->type ?? 'user';
        $data['experience'] = $request->experience;
        $data['speciality'] = $request->speciality;

        $userData = [
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make($request->password),
            'phone' => $request->phone,
            'gender' => $request->gender,
            'type' => $request->type ?? 'user',
            'experience' => $request->experience,
            'speciality' => $request->speciality,
        ];

        // Handle image upload
        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $imageName = time().'.'.$image->getClientOriginalExtension();
            
            // Create directory if it doesn't exist
            $destinationPath = public_path('uploads/user');
            if (!file_exists($destinationPath)) {
                mkdir($destinationPath, 0777, true);
            }
            
            // Move file to public directory
            $image->move($destinationPath, $imageName);
            $userData['image'] = 'uploads/user/'.$imageName; // Store relative path
        }

        $user = User::create($userData);
        $role = $request->type == 'chef' ? 'chef' : 'user'; // Simplified role assignment
        $user->assignRole($role);

        $tokenResult = $user->createToken('Personal Access Token');
        $token = $tokenResult->token;

        if ($request->remember_me) {
            $token->expires_at = Carbon::now()->addWeeks(1);
        }

        $token->save();
        $status_code = 200;
        $data['status'] = 'success';
        $data['message'] = 'User Registered Successfully!';
        $data['image'] = isset($user->image) ? asset($user->image) : null;
        $data['id'] = $user->id;
        $data['token'] = $tokenResult->accessToken;
        $data['type'] = $role;
        
        return response()->json($data, $status_code);
    }

    public function verifyOtp(Request $request)
    {
        $user = $this->authRepository->userVerifyOtp();
        return $user;
    }

    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required|string|min:8',
        ]);
        $token = $request->bearerToken();
        $data['status'] = "error";
        $data['message'] = "";
        $data['email'] = "";
        $data['name'] = "";
        $data['password'] = "";
        $data['image'] = "";
        $data['type'] = "";
        // $data['access_token'] = "";
        $status_code = "";
        if ($validator->fails()) {
            $data['message'] = 'Validation error';
            $error_message = $validator->messages();
            if ($error_message->first('email')) {
                $data['email'] = $error_message->first('email');
            }
            if ($error_message->first('password')) {
                $data['password'] = $error_message->first('password');
            }
            $status_code = 400;
            return response()->json($data, $status_code);
        }
        if (!Auth::attempt($request->only('email', 'password'))) {
            $data['message'] = 'Unauthorized';
            return response()->json($data, 401);
        }
        $data['email'] = $request['email'];
        $user = User::where(['email' => $request->email])->first();
        if (!$user) {
            $data['message'] = 'User not found or does not exists';
            return response()->json($data, 404);
        }
        
        $tokenResult = $user->createToken('Personal Access Token');
        $token = $tokenResult->token;

        if ($request->remember_me) {
            $token->expires_at = Carbon::now()->addWeeks(1);
        }

        $token->save();

        $data['name'] = $user->name ?? "";
        $data['id'] = $user->id;
        $data['image'] = $user->image ?? url('/images/default-profile.jpg');
        $data['status'] = 'success';
        $data['message'] = 'you are loggedin';
        // $data['access_token'] = $token;
        $data['token'] = $tokenResult->accessToken;
        $data['type'] = $user->type;
        return response()->json($data, 200);
    }

    public function resendOtp(Request $request)
    {
        $user = $this->authRepository->userResendOtp();
        return $user;
    }

    public function forgotPassword(Request $request)
    {
        $user = $this->authRepository->userForgotPassword();
        return $user;
    }

    public function resetPassword(Request $request)
    {
        $user = $this->authRepository->userResetPassword();
        return $user;
    }

    public function updatePassword(Request $request)
    {
        $user = $this->authRepository->userUpdatePassword();
        return $user;
    }

    public function logout(Request $request)
    {
        $user = $this->authRepository->userLogout();
        return $user;
    }

    public function getProfile(Request $request)
    {
        $user = $this->authRepository->userGetProfile();
        return $user;
    }

    public function updateProfile(UpdateProfileRequest $authRepository)
    {
        $user = $this->authRepository->userUpdateProfile();
        return $user;
    }

    public function deleteAccount(Request $request)
    {
        $user = $this->authRepository->userDeleteAccount();
        return $user;
    }

    public function saveUserDeviceToken(Request $request)
    {
        $user = $this->authRepository->userDeviceToken();
        return $user;
    }

    public function googleAuth(GoogleAuthRequest $authRepository)
    {
        $user = $this->authRepository->userGoogleAuth();
        return $user;
    }

    public function facebookAuth(FacebookAuthRequest $authRepository)
    {
        $user = $this->authRepository->userFacebookAuth();
        return $user;
    }

    public function appleAuth(AppleAuthRequest $authRepository)
    {
        $user = $this->authRepository->userAppleAuth();
        return $user;
    }
}
