<?php

namespace App\Http\Controllers\Api\ChiefPost;

use App\Models\User;
use App\Models\ChiefCommentReply;
use App\Models\ChiefComment;
use App\Models\ChiefPostReaction;
use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\ChiefPost;
use App\Models\ChiefPostImage;
use App\Http\Traits\ApiResponse;
use Spatie\Permission\Traits\HasRoles;
use Carbon\Carbon;
use Auth;
use DB;

class ChiefPostController extends Controller
{
    use ApiResponse, HasRoles;

    // public function listing(Request $request)
    // {
    //     $posts  = ChiefPost::where('status', '0')->with(['user:id,name,image', 'images:id,post_id,image', 'comments.user:id,name,image', 'comments.replies.user:id,name,image'])
    //     ->select('id','user_id', 'date', 'status', 'description')
    //     ->withCount([
    //         'reactions as likes_count' => fn($q) => $q->where('type', 'like'),
    //         'reactions as dislikes_count' => fn($q) => $q->where('type', 'dislike')
    //     ])
    //     ->latest()
    //     ->get();
            
    //     return response()->json([
    //         'status' => 1,
    //         'message' => 'Post Listing successfully',
    //         'data' => $posts,
    //     ], 200);
    // }
    
    public function listing(Request $request)
    {
        // Get latest 5 unique chefs who created posts
        $latestChefs = ChiefPost::where('status', '0')
            ->with(['user:id,name,image'])
            ->select('id', 'user_id', 'date')
            ->latest()
            ->take(5)
            ->get()
            ->map(function ($post) {
                return [
                    'name' => $post->user->name,
                    'image' => $post->user->image 
                        ? asset(str_replace('public/', 'public/', $post->user->image)) 
                        : asset('public/uploads/default/default.png'),
                    'date_ago' => Carbon::parse($post->date)->diffForHumans()
                ];
            })
            ->unique('name')
            ->values();
    
        $posts = ChiefPost::where('status', '0')
            ->with(['user:id,name,image', 'images:id,post_id,image', 'comments.user:id,name,image', 'comments.replies.user:id,name,image'])
            ->select('id', 'user_id', 'date', 'status', 'description')
            ->withCount([
                'reactions as likes_count' => fn($q) => $q->where('type', 'like'),
                'reactions as dislikes_count' => fn($q) => $q->where('type', 'dislike')
            ])
            ->latest()
            ->get();
                
        return response()->json([
            'status' => 1,
            'message' => 'Chief Posts Listing successfully',
            'data' => [
                'listing' => $posts,
                'latestChefs' => $latestChefs
            ],
        ], 200);
    }

    public function create(Request $request)
    {
        if (auth()->user()->type != 'chef') {
            return response()->json([
                'status' => 'Error',
                'message' => 'Unauthorized - Only chiefs can create posts',
                'data' => null
            ], 403);
        }

        $validator = Validator::make($request->all(), [
            'description' => 'nullable|string',
            'images' => 'nullable|array',
            'images.*' => 'image|mimes:jpeg,png,jpg,gif|max:2048'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        try {
            $post = ChiefPost::create([
                'user_id' => auth()->id(),
                'description' => $request->description,
                'status' => '0',
                'date' => now(),
            ]);

            if ($request->hasFile('images')) {
                foreach ($request->file('images') as $image) {
                    $path = 'public/uploads/chief_post/' . $image->hashName();
                    $image->move(public_path('uploads/chief_post'), $image->hashName());
                    
                    ChiefPostImage::create([
                        'post_id' => $post->id,
                        'image' => $path
                    ]);
                }
            }
            $post->date = $post->date->format('M j, Y');
            
            return response()->json([
                'status' => 'Success',
                'message' => 'Post created successfully',
                'data' => $post->load('images')
            ], 201);

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Failed to create post',
                'data' => $e->getMessage()
            ], 500);
        }
    }

    public function delete(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id' => 'required|exists:chief_posts,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        try {
            $post = ChiefPost::findOrFail($request->post_id);
            
            if ($post->user_id != auth()->id()) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Unauthorized: You can only delete your own posts'
                ], 403);
            }

            DB::transaction(function () use ($post) {
                $images = ChiefPostImage::where('post_id', $post->id)->get();
                foreach ($images as $image) {
                    $filePath = str_replace('public/', storage_path('app/public/'), $image->image);
                    if (file_exists($filePath)) {
                        unlink($filePath);
                    }
                    $image->delete();
                }
                
                ChiefPostReaction::where('post_id', $post->id)->delete();
                
                $comments = ChiefComment::where('post_id', $post->id)->get();
                
                foreach ($comments as $comment) {
                    ChiefCommentReply::where('comment_id', $comment->id)->delete();
                }
                
                $comments->each->delete();
                
                $post->delete();
            });

            return response()->json([
                'status' => 'success',
                'message' => 'Post and all related data deleted successfully',
                'data' => null
            ]);

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Failed to delete post',
                'data' => $e->getMessage()
            ], 500);
        }
    }

    public function reaction(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type' => 'required|in:like,dislike',
            'post_id' => 'required|exists:chief_posts,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        try {
            $post = ChiefPost::findOrFail($request->post_id);
            $userId = auth()->id();
            
            $existingReaction = ChiefPostReaction::where('post_id', $request->post_id)
                ->where('user_id', $userId)
                ->first();

            $action = 'added';
            $message = 'Reaction added successfully';

            if ($existingReaction) {
                if ($existingReaction->type === $request->type) {
                    // Same reaction - remove it
                    $existingReaction->delete();
                    $action = 'removed';
                    $message = 'Reaction removed successfully';
                } else {
                    // Different reaction - update it
                    $existingReaction->update(['type' => $request->type]);
                    $action = 'updated';
                    $message = 'Reaction updated successfully';
                }
            } else {
                // New reaction - create it
                ChiefPostReaction::create([
                    'post_id' => $request->post_id,
                    'user_id' => $userId,
                    'type' => $request->type
                ]);
            }

            // Get updated counts
            $reactionCounts = ChiefPostReaction::where('post_id', $request->post_id)
                ->selectRaw('sum(case when type = "like" then 1 else 0 end) as likes_count')
                ->selectRaw('sum(case when type = "dislike" then 1 else 0 end) as dislikes_count')
                ->first();

            return response()->json([
                'status' => 'Success',
                'message' => $message,
                'data' => [
                    'action' => $action,
                    'likes_count' => (int)$reactionCounts->likes_count,
                    'dislikes_count' => (int)$reactionCounts->dislikes_count
                ]
            ]);

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Failed to process reaction',
                'data' => $e->getMessage()
            ], 500);
        }
    }

    public function commentStore(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'content' => 'required|string',
            'post_id' => 'required|exists:chief_posts,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        $comment = ChiefComment::create([
            'post_id' => $request->post_id,
            'user_id' => auth()->id(),
            'content' => $request->content
        ]);

        return response()->json([
            'status' => 'Success',
            'message' => 'Comment created successfully',
            'data' => $comment->load('user:id,name,image')
        ], 201);

    }

    public function commentDelete(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'comment_id' => 'required|exists:chief_comments,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        $comment = ChiefComment::findOrFail($request->comment_id);

        if ($comment->user_id != auth()->id()) {
            return response()->json([
                'status' => 'error',
                'message' => 'Unauthorized: You can only delete your own comments'
            ], 403);
        }
        DB::table('chief_comment_replies')->where('comment_id', $comment->id)->delete();
        $comment->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Comment deleted successfully',
            'data' => $comment
        ]);
    }

    public function replyStore(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'comment_id' => 'required|exists:chief_comments,id',
            // 'post_id' => 'required|exists:chief_posts,id',
            'parent_id' => [
                'nullable',
                'integer',
                function ($attribute, $value, $fail) use ($request) {
                    if ($value && !ChiefCommentReply::where('id', $value)->where('comment_id', $request->comment_id)->exists()) {
                        $fail('The selected parent reply does not exist or belongs to a different comment.');
                    }
                }
            ],
            // 'post_id' => 'required|exists:posts,id',
            'content' => 'required|string'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        $replyData = [
            'comment_id' => $request->comment_id,
            'post_id' => $request->post_id,
            'user_id' => auth()->id(),
            'content' => $request->content
        ];

        // if ($request->filled('parent_id')) {
        //     $replyData['parent_id'] = $request->parent_id;
        // }
    
        $reply = ChiefCommentReply::create($replyData);

        return response()->json([
            'status' => 'Success',
            'message' => 'Reply created successfully',
            'data' => $reply->load('parent.user:id,name,image', 'comment:id,post_id,user_id,content', 'user:id,name,image')
        ], 201);
    }

    public function replyDelete(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'reply_id' => 'required|exists:chief_comment_replies,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }
        $reply = ChiefCommentReply::findOrFail($request->reply_id);
        if ($reply->user_id !== auth()->id()) {
            return response()->json([
                'status' => 'error',
                'message' => 'Unauthorized: You can only delete your own replies'
            ], 403);
        }

        $reply->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Reply deleted successfully',
            'data' => $reply
        ]);
    }
}
