<?php

namespace App\Http\Controllers\Api\Review;

use App\Models\User;
use App\Models\ReviewReaction;
use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Review;
use App\Models\ReviewImage;
use App\Http\Traits\ApiResponse;
use Spatie\Permission\Traits\HasRoles;
use Carbon\Carbon;
use Auth;
use DB;

class ReviewController extends Controller
{
    use ApiResponse, HasRoles;

    // public function listing(Request $request)
    // {
    //     $reviews = Review::where('status', '0')->with(['user:id,name,image', 'images:id,review_id,image'])
    //     ->select('id','user_id', 'rating', 'date', 'status', 'description')
    //     ->latest()->get()
    //     ->map(function ($review) {
    //         $review->reaction_counts = $review->getReactionCounts();
    //         return $review;
    //     });
            
    //     return response()->json([
    //         'status' => 'Success',
    //         'message' => 'Reviews Listing successfully',
    //         'data' => $reviews,
    //     ], 201);
    // }
    
    public function listing(Request $request)
    {
        // Get the latest 5 reviews with 5-star rating
        $userLatestReview = Review::where('status', '0')
            ->where('rating', '5')
            ->with(['user:id,name,image'])
            ->select('id', 'user_id', 'date')
            ->latest()
            ->take(5)
            ->get()
            ->map(function ($review) {
                return [
                    'name' => $review->user->name,
                    'image' => $review->user->image 
                        ? asset(str_replace('public/', 'public/', $review->user->image)) 
                        : asset('public/uploads/default/default.png'),
                    'date' => $review->date->diffForHumans() // ago format
                ];
            });
        
        // Get all reviews with status 0
        $reviews = Review::where('status', '0')
            ->with(['user:id,name,image', 'images:id,review_id,image'])
            ->select('id', 'user_id', 'rating', 'date', 'status', 'description')
            ->latest()
            ->get()
            ->map(function ($review) {
                $review->reaction_counts = $review->getReactionCounts();
                return $review;
            });
            
        return response()->json([
            'status' => 'Success',
            'message' => 'Reviews Listing successfully',
            'data' => [
                'listing' => $reviews,
                'userLatestReview' => $userLatestReview
            ],
        ], 200);
    }

    public function create(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'rating' => 'required|integer|between:1,5',
            'description' => 'required',
            'images' => 'nullable|array',
            'images.*' => 'nullable'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        try {
            $review = Review::create([
                'user_id' => auth()->id(),
                'rating' => $request->rating,
                'description' => $request->description,
                'status' => '0',
                'date' => now(),
            ]);

            if ($request->hasFile('images')) {
                foreach ($request->file('images') as $image) {
                    $path = 'public/uploads/review/' . $image->hashName();
                    $image->move(public_path('uploads/review'), $image->hashName());
                    
                    ReviewImage::create([
                        'review_id' => $review->id,
                        'image' => $path
                    ]);
                }
            }
            $review->date = $review->date->format('M j, Y');
            
            return response()->json([
                'status' => 'Success',
                'message' => 'Review created successfully',
                'data' => $review->load('images')
            ], 201);

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Failed to create review',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function reaction(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'type' => 'required|in:thanks,helpful,love_this,oh_no',
            'review_id' => 'required|exists:reviews,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'errors' => $validator->errors()
            ], 422);
        }
        
        if (!auth()->check()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Unauthorized'
            ], 401);
        }

        try {
            $review = Review::findOrFail($request->review_id);
            $userId = auth()->id();
            $action = 'added';
            $message = 'Reaction added successfully';

            $existingReaction = ReviewReaction::where('review_id', $request->review_id)
                ->where('user_id', $userId)
                ->first();

            if ($existingReaction) {
                if ($existingReaction->type === $request->type) {
                    $existingReaction->delete();
                    $action = 'removed';
                    $message = 'Reaction removed successfully';
                    $reaction = null;
                } else {
                    $existingReaction->update(['type' => $request->type]);
                    $action = 'updated';
                    $message = 'Reaction updated successfully';
                    $reaction = $existingReaction;
                }
            } else {
                $reaction = ReviewReaction::create([
                    'review_id' => $request->review_id,
                    'user_id' => $userId,
                    'type' => $request->type
                ]);
            }

            $reactionCounts = $review->getReactionCounts();
            
            $responseData = [
                'status' => 'Success',
                'message' => $message,
                'data' => [
                    'action' => $action,
                    'reaction_counts' => $reactionCounts
                ]
            ];

            // if ($reaction) {
            //     $responseData['data']['reaction'] = [
            //         'id' => $reaction->id,
            //         'type' => $reaction->type,
            //         'user_id' => $reaction->user_id,
            //         'review_id' => $reaction->review_id
            //     ];
            // }

            return response()->json($responseData);

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Failed to process reaction',
                'error' => $e->getMessage()
            ], 500);
        }
    }
    
    public function delete(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'review_id' => 'required|exists:reviews,id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Validation error',
                'data' => $validator->errors()
            ], 422);
        }

        try {
            $review = Review::findOrFail($request->review_id);
            
            if ($review->user_id != auth()->id()) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Unauthorized: You can only delete your own reviews'
                ], 403);
            }

            DB::transaction(function () use ($review) {
                $images = ReviewImage::where('review_id', $review->id)->get();
                foreach ($images as $image) {
                    $filePath = str_replace('public/', storage_path('app/public/'), $image->image);
                    if (file_exists($filePath)) {
                        unlink($filePath);
                    }
                    $image->delete();
                }
                
                // dd($images);

                ReviewReaction::where('review_id', $review->id)->delete();
                
                $review->delete();
            });

            return response()->json([
                'status' => 'success',
                'message' => 'Review and all related data deleted successfully',
                'data' => null
            ]);

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'Error',
                'message' => 'Failed to delete Review',
                'data' => $e->getMessage()
            ], 500);
        }
    }


    // public function create0(Request $request)
    // {
    //     $validator = Validator::make($request->all(), [
    //         'rating' => 'required|integer|between:1,5',
    //         'description' => 'required|string|max:1000',
    //         'images' => 'nullable|array',
    //     ]);

    //     if ($validator->fails()) {
    //         return response()->json([
    //             'status' => 0,
    //             'message' => 'Validation error',
    //             'data' => $validator->errors()
    //         ], 422);
    //     }

    //     try {
    //         $review = Review::create([
    //             'user_id' => auth()->id(),
    //             'rating' => $request->rating,
    //             'description' => $request->description,
    //             'date' => now(),
    //         ]);

    //         if ($request->has('images') && is_array($request->images)) {
    //             foreach ($request->images as $base64Image) {
    //                 // Extract image data from base64 string
    //                 preg_match('/^data:image\/(\w+);base64,/', $base64Image, $matches);
    //                 $extension = $matches[1] ?? 'png';
    //                 $imageData = substr($base64Image, strpos($base64Image, ',') + 1);
                    
    //                 // Generate unique filename
    //                 $filename = uniqid() . '.' . $extension;
    //                 $path = 'uploads/review/' . $filename;
                    
    //                 // Save image to storage
    //                 File::put(public_path($path), base64_decode($imageData));
    
    //                 ReviewImage::create([
    //                     'review_id' => $review->id,
    //                     'image' => $path
    //                 ]);
    //             }
    //         }

    //         $review->date = $review->date->format('M j, Y');
            
    //         return response()->json([
    //             'status' => 1,
    //             'message' => 'Review created successfully',
    //             'data' => $review->load('images')
    //         ], 201);

    //     } catch (\Exception $e) {
    //         return response()->json([
    //             'status' => 0,
    //             'message' => 'Failed to create review',
    //             'error' => $e->getMessage()
    //         ], 500);
    //     }
    // }

}
